﻿using Microsoft.AspNetCore.Mvc;
using Syncfusion.DocIO;
using Syncfusion.DocIO.DLS;
using Microsoft.AspNetCore.Hosting;
using System.Collections;
using System.Collections.Generic;
using System;

namespace DocIOApplication.Controllers
{
    public class HomeController : Controller
    {
        private readonly IHostingEnvironment _hostingEnvironment;
        public HomeController(IHostingEnvironment hostingEnvironment)
        {
            _hostingEnvironment = hostingEnvironment;
        }
        public ActionResult Index(string button1, string Message)
        {
            if (button1 == null)
                return View();

            //Gets file path for template Word document.
            string rootPath = _hostingEnvironment.WebRootPath;
            //Opens an existing document from file system through constructor of WordDocument class
            WordDocument document = new WordDocument(rootPath + @"\..\Template.docx");
            // Field names and values to execute Mail merge
            string[] fieldNames = document.MailMerge.GetMergeFieldNames();
            string[] fieldValues = new string[fieldNames.Length];
            //Process Mail merge only MergeField present in template
            if (fieldNames.Length > 0)
            {
                //Visit all document entities
                foreach (var item in document.Visit())
                {
                    //If the entity is MergeField
                    if (item.EntityType == EntityType.MergeField)
                    {
                        fieldValues.SetValue((item as WMergeField).Text, Array.IndexOf(fieldNames, (item as WMergeField).FieldName));
                    }
                }
                //Performs the mail merge.
                document.MailMerge.Execute(fieldNames, fieldValues);
            }
            //Saves and closes the WordDocument instance
            document.Save("Sample.docx", FormatType.Docx);
            document.Close();

            return View();
        }
        public IActionResult About()
        {
            ViewData["Message"] = "Your application description page.";

            return View();
        }

        public IActionResult Contact()
        {
            ViewData["Message"] = "Your contact page.";

            return View();
        }

        public IActionResult Error()
        {
            return View();
        }
    }
    #region ExtendedClass
    public static class DocIOExtensions
    {
        public static IEnumerable<IEntity> Visit(this ICompositeEntity entity)
        {
            var entities = new Queue<IEntity>(new IEntity[] { entity });
            while (entities.Count > 0)
            {
                var e = entities.Dequeue();
                yield return e;
                if (e is ICompositeEntity)
                {
                    foreach (IEntity childEntity in ((ICompositeEntity)e).ChildEntities)
                    {
                        entities.Enqueue(childEntity);
                    }
                }
            }
        }
    }
    #endregion
}
